/*******************************************************************************
 * Copyright (c) 2004 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.contribution.visualiser.google;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.contribution.visualiser.VisualiserPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

/**
 * Provides the view in which search terms can be entered.
 */
public class SearchView extends ViewPart {

	public static final String PREF_KEY = "org.eclipse.contribution.visualiser.google.clientkey";

	private Text inputField, keyField;

	private QueryEngine engine = new QueryEngine();

	public SearchView() {
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		Group group = new Group(parent, SWT.NONE);
		group.setLayout(new GridLayout(2, false));
		GridData gd = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
		group.setLayoutData(gd);
		group.setText("Google results visualization");

		Label prompt = new Label(group, SWT.NONE);
		prompt.setText("Enter search string:");

		new Label(group, SWT.NONE);

		inputField = new Text(group, SWT.SINGLE | SWT.BORDER);
		gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		inputField.setLayoutData(gd);
		inputField.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent event) {
				performQuery(inputField.getText(), keyField.getText());
			}
		});

		Button go = new Button(group, SWT.PUSH);
		go.setText("Perform Search");
		go.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				performQuery(inputField.getText(), keyField.getText());
			}
		});

		prompt = new Label(group, SWT.NONE);
		gd = new GridData();
		gd.horizontalSpan = 2;
		prompt.setLayoutData(gd);
		prompt.setText("Enter license key:");

		keyField = new Text(group, SWT.SINGLE | SWT.BORDER);
		gd = new GridData();
		gd.horizontalSpan = 2;
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = SWT.FILL;
		keyField.setLayoutData(gd);
		keyField.setText(retrieveKey());
	}

	/**
	 * Execute the given query and update the visualiser with the results
	 * 
	 * @param txt
	 */
	private void performQuery(final String txt, final String key) {
		if (key == null || key.length() == 0) {
			MessageDialog
					.openError(
							GoogleVisPlugin.getDefault().getWorkbench()
									.getActiveWorkbenchWindow().getShell(),
							"Missing license key",
							"You need to enter your personal license key which is required to use the Google Web APIs. You can obtain a key by creating an account at http://www.google.com/apis/");
			keyField.setFocus();
			return;
		}
		if (txt == null || txt.length() == 0) {
			MessageDialog.openError(GoogleVisPlugin.getDefault().getWorkbench()
					.getActiveWorkbenchWindow().getShell(),
					"No search keywords",
					"You need to enter some keywords for the search.");
			inputField.setFocus();
			return;
		}
		storeKey(key);
		engine.setKey(key);
		IRunnableWithProgress runnable = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor)
					throws InvocationTargetException {
				try {
					List results = engine.doQuery(txt, monitor);
					GoogleContentProvider.setResults(results);
					VisualiserPlugin.refresh();
				} catch (SearchException se) {
					throw new InvocationTargetException(se);
				}
			}
		};
		try {
			new ProgressMonitorDialog(getViewSite().getShell()).run(true, true,
					runnable);
		} catch (InvocationTargetException e) {
			MessageDialog.openError(getViewSite().getShell(), "Search failed",
					"The search failed due to the following problem:"
							+ System.getProperty("line.separator")
							+ e.getTargetException().getMessage());
		} catch (InterruptedException e) {
			// the search was cancelled by the user
		}
	}

	private void storeKey(String key) {
		IPreferenceStore store = GoogleVisPlugin.getDefault()
				.getPreferenceStore();
		store.setValue(PREF_KEY, key);
	}

	private String retrieveKey() {
		IPreferenceStore store = GoogleVisPlugin.getDefault()
				.getPreferenceStore();
		return store.getString(PREF_KEY);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		inputField.setFocus();
	}
}